"use strict";
var __createBinding = (this && this.__createBinding) || (Object.create ? (function(o, m, k, k2) {
    if (k2 === undefined) k2 = k;
    Object.defineProperty(o, k2, { enumerable: true, get: function() { return m[k]; } });
}) : (function(o, m, k, k2) {
    if (k2 === undefined) k2 = k;
    o[k2] = m[k];
}));
var __setModuleDefault = (this && this.__setModuleDefault) || (Object.create ? (function(o, v) {
    Object.defineProperty(o, "default", { enumerable: true, value: v });
}) : function(o, v) {
    o["default"] = v;
});
var __importStar = (this && this.__importStar) || function (mod) {
    if (mod && mod.__esModule) return mod;
    var result = {};
    if (mod != null) for (var k in mod) if (k !== "default" && Object.prototype.hasOwnProperty.call(mod, k)) __createBinding(result, mod, k);
    __setModuleDefault(result, mod);
    return result;
};
var __awaiter = (this && this.__awaiter) || function (thisArg, _arguments, P, generator) {
    function adopt(value) { return value instanceof P ? value : new P(function (resolve) { resolve(value); }); }
    return new (P || (P = Promise))(function (resolve, reject) {
        function fulfilled(value) { try { step(generator.next(value)); } catch (e) { reject(e); } }
        function rejected(value) { try { step(generator["throw"](value)); } catch (e) { reject(e); } }
        function step(result) { result.done ? resolve(result.value) : adopt(result.value).then(fulfilled, rejected); }
        step((generator = generator.apply(thisArg, _arguments || [])).next());
    });
};
var __importDefault = (this && this.__importDefault) || function (mod) {
    return (mod && mod.__esModule) ? mod : { "default": mod };
};
Object.defineProperty(exports, "__esModule", { value: true });
const radash_1 = __importDefault(require("radash"));
const pulumi = __importStar(require("@pulumi/pulumi"));
const aws = __importStar(require("@pulumi/aws"));
const fs_extra_1 = __importDefault(require("fs-extra"));
const pulumi_aws_lambda_api_1 = require("@exobase/pulumi-aws-lambda-api");
const builds_1 = require("@exobase/builds");
const path_1 = __importDefault(require("path"));
const cmdish_1 = __importDefault(require("cmdish"));
const webpack_1 = __importDefault(require("webpack"));
const terser_webpack_plugin_1 = __importDefault(require("terser-webpack-plugin"));
const main = ({ workingDir, }) => __awaiter(void 0, void 0, void 0, function* () {
    //
    //  READ PROJECT CONFIG
    //
    const context = (yield fs_extra_1.default.readJSON(path_1.default.join(workingDir, "context.json")));
    const { platform, service, deployment } = context;
    const config = deployment.config.stack;
    console.log('x--> CONTEXT:');
    console.log(JSON.stringify(context, null, 2));
    //
    //  SETUP PROVIDER
    //
    const provider = new aws.Provider("aws", {
        secretKey: platform.providers.aws.accessKeySecret,
        accessKey: platform.providers.aws.accessKeyId,
        region: platform.providers.aws.region,
    });
    //
    //  READ FUNCTIONS
    //
    const functions = (0, builds_1.getFunctionMap)({
        path: path_1.default.resolve(workingDir, 'source'),
        ext: 'ts'
    });
    console.log('x--> FUNCTIONS:');
    console.log(functions);
    //
    //  EXECUTE BUILD
    //
    yield (0, cmdish_1.default)('nvm use 16', { cwd: path_1.default.resolve(workingDir, 'source') });
    yield (0, cmdish_1.default)('yarn', { cwd: path_1.default.resolve(workingDir, 'source') });
    yield executeBuild(functions, path_1.default.resolve(workingDir, 'source'));
    //
    //  CREATE API/LAMBDA RESOURCES
    //
    const envVarDict = deployment.config.environmentVariables.reduce((acc, ev) => (Object.assign(Object.assign({}, acc), { [ev.name]: ev.value })), {});
    const api = new pulumi_aws_lambda_api_1.AWSLambdaAPI(radash_1.default.dashCase(service.name), {
        sourceDir: path_1.default.join(workingDir, "source"),
        sourceExt: "ts",
        runtime: "nodejs14.x",
        functions,
        getZip: (func) => {
            return new pulumi.asset.FileArchive(path_1.default.resolve(workingDir, 'source', 'build', 'modules', func.module, `${func.function}.zip`));
        },
        getHandler: (func) => `${func.function}.default`,
        timeout: toNumber(config.timeout),
        memory: toNumber(config.memory),
        environmentVariables: Object.assign(Object.assign({}, envVarDict), { EXOBASE_PLATFORM: platform.name, EXOBASE_SERVICE: service.name }),
        domain: service.domain,
    }, { provider });
    return {
        url: service.domain ? service.domain.fqd : api.api.url,
    };
});
const toNumber = (value) => {
    if (radash_1.default.isString(value))
        return parseInt(value);
    return value;
};
const executeBuild = (functions, sourceDir) => __awaiter(void 0, void 0, void 0, function* () {
    yield (0, cmdish_1.default)('rm -rf build', { cwd: sourceDir });
    const execute = (func) => __awaiter(void 0, void 0, void 0, function* () {
        console.log(`processing: ${func.module}/${func.function}.js`);
        yield compile(func, sourceDir);
        console.log(`compiled: ${func.module}/${func.function}.js`);
        yield zip(func, sourceDir);
        console.log(`zipped: ${func.module}/${func.function}.js -> ${func.module}/${func.function}.zip`);
    });
    yield Promise.allSettled(functions.map(f => execute(f).catch((err) => {
        console.error(err);
        throw err;
    })));
});
const compile = (func, sourceDir) => __awaiter(void 0, void 0, void 0, function* () {
    yield new Promise((res, rej) => {
        var _a;
        (0, webpack_1.default)({
            entry: [path_1.default.resolve(sourceDir, `src/modules/${func.module}/${func.function}.ts`)],
            mode: (_a = process.env.NODE_ENV) !== null && _a !== void 0 ? _a : 'production',
            target: 'node',
            output: {
                path: path_1.default.resolve(sourceDir, 'build', 'modules', func.module),
                filename: `${func.function}.js`
            },
            resolve: {
                extensions: ['.ts', '.js']
            },
            module: {
                rules: [
                    {
                        test: /\.ts$/,
                        use: ['ts-loader']
                    }
                ]
            },
            optimization: {
                minimizer: [
                    new terser_webpack_plugin_1.default({
                        extractComments: false
                    })
                ]
            }
        }, (err, stats) => {
            console.log('x--> WEBPACK COMPLETE:');
            console.log(err);
            console.log(stats);
            if (err || stats.hasErrors())
                rej(err !== null && err !== void 0 ? err : { message: 'Webpack stats has error' });
            else
                res();
        });
    });
});
const zip = (func, sourceDir) => __awaiter(void 0, void 0, void 0, function* () {
    yield (0, cmdish_1.default)(`zip -q ${func.function}.zip ${func.function}.js`, {
        cwd: path_1.default.resolve(sourceDir, 'build', 'modules', func.module)
    });
});
exports.default = main;
//# sourceMappingURL=index.js.map